# This is an example of how we see 
# the package work. The functions listed here
# are probably teh only ones that should be exposed, ie documented.
# others should br prepended with a double underscore
#  
# The cognet directory has the following "modules"
# which are seprate .py files containing clases and functions
from mpi4py.futures import MPIPoolExecutor
import sys
sys.path.append("../../cognet")

from quasinet.qnet import qdistance
from cognet import cognet

from dataFormatter import dataFormatter
from model import model
import pandas as pd
import numpy as np

yr = '2018'
POLEFILE='../examples_data/polar_vectors.csv'
QPATH='../examples_data/gss_'+yr+'.joblib'
IMMUTABLE_FILE='../examples_data/immutable.csv'
GSSDATA = '../examples_data/gss_'+yr+'.csv'

commandline_test = True
#train data object
data = dataFormatter(samples=GSSDATA,
                test_size=0.5)
features,samples = data.train()

im_vars_df = pd.read_csv(IMMUTABLE_FILE, names=['vars'])
im_vars_list = im_vars_df.vars.to_list()
mutable_vars, immutable_vars = data.mutable_variables(immutable_list=im_vars_list)
mutable_vars, immutable_vars = data.mutable_variables(IMMUTABLE_FILE=IMMUTABLE_FILE)

model_ = model()
model_.load("../examples_data/gss_2018.joblib")

Cg = cognet()
Cg.load_from_model(model_, samples_file='../examples_data/gss_2018.csv')

# produce stats on how many column names actually match
stats = Cg.set_poles(POLEFILE,steps=2)
Cg.num_qsamples = 5

if commandline_test:
    print("testing mpiexex print error")
    Cg.set_nsamples(100)
#distance_matrix=Cg.distfunc_multiples("distfunc_multiples_testing.csv")

if __name__ == '__main__':

    with MPIPoolExecutor() as executor:
        result = executor.map(Cg.distfunc_line, range(len(Cg.samples)))
        print(result)
        pd.DataFrame(result).to_csv('../examples_results/distfunc_mpitest_computenode.csv',index=None,header=None)


# This is an example of how we see 
# the package work. The functions listed here
# are probably teh only ones that should be exposed, ie documented.
# others should br prepended with a double underscore
#  
# The cognet directory has the following "modules"
# which are seprate .py files containing clases and functions
# The modules are cognet.py, dataFormatter.py, model.py, util.py, viz.py
# we will write the viz.py later.
import sys
sys.path.append("../")

from cognet.cognet import cognet
print(cognet)
from cognet.dataFormatter import dataFormatter
from cognet.model import model 
#import cognet.util
import pandas as pd
#from cognet.viz import distance_contour 

yr = '2018'
GSS_dir = '../../creed2_/GSS/data/processed_data/'
POLEFILE='../../creed2_/GSS/polar_vectors.csv'
GSS_FEATURES_BY_YEAR='../../creed2_/GSS/data/features/features_by_year_GSS.csv'
GSSDATA=GSS_dir+'gss_'+yr+'.csv'
QPATH='../../creed2_/GSS/Qnets/gss_'+yr+'.joblib'
IMMUTABLE_FILE='../../creed2_/GSS/immutable.csv'

test_dataFormatter = True
test_model = False
if test_dataFormatter:
        data = dataFormatter(samples=GSSDATA,
                        test_size=0.5)
        # load the sample data
        # have option for test/train split
        # make checks to ensure we will not bark at qnet construction 
        # data.train() returns traininh data
        # data.test() returns test data
        
        features,samples = data.train()
        features,samples = data.train()
        # we can set mutable and immutable vars from list or file
        im_vars_df = pd.read_csv(IMMUTABLE_FILE, names=['vars'])
        im_vars_list = im_vars_df.vars.to_list()
        mutable_vars, immutable_vars = data.set_vars(immutable_list=im_vars_list)
        mutable_vars, immutable_vars = data.set_vars(IMMUTABLE_FILE=IMMUTABLE_FILE)


        testing = False
        # can either input features and samples directly, or infer from data obj
        
        model_ = model()
        model_.fit(data_obj=data)
        # qnet construction parameters 
        # infer qnet
        
        if test_model:
                model_.export_dot("tmp_dot_modelclass.dot",
                                generate_trees=True)
                model_.save("tmp_nodelclass.joblib")
                model_.load("tmp_nodelclass.joblib")
        
        # set some paramaters in instantiating cognet class 
        # also setup Dnull, and nullbaseFreq
        # if loading from model obj, no need to load_data, otherwise, load_data
         
        Cg = cognet()
        Cg.load_from_model(model_)
        
        # distance calculation for individual samples    
        # we have a nsteps parameter (for sample 1 and sample2)
        # which qsamples the sample1 and sample2 if set before
        # computing distance. Note qsampling must only 
        # change mutable varaibles, so need to compute base-freq
        distance = Cg.distance(samples[1],samples[3],nsteps1=5, nsteps2=5)
        #distance = Cg.distance(data.samples[0],data.samples[1])
        #distance = Cg.distance(data.test[0],data.test[1])

        # produce stats on how many column names actually match
        stats = Cg.set_poles(POLEFILE,steps)

        # compute polar distance matrix
        dmatrix = Cg.polar_separation(nsteps=0)

        # the following are for single samples
        #------------------
        dissonance_array = Cg.dissonance(sample)
        rederr,r_prob,rand_err = Cg.randomMaskReconstruction(sample)
        #ideology_index = Cg.compute_DLI_sample(3)
        local_dispersion = Cg.compute_DLI_sample(3)[4:]
        # compute distance from each pole
        array_distances = Cg.polarDistance(sample)
        #-------------------

        #the following must use parallelization
        # next one must use mpi and hence will not run
        # with mpi without maybe a seprate script.
        # But look here: https://stackoverflow.com/questions/25772289/python-multiprocessing-within-mpi
        distance_matrix=Cg.qdistance_matrix(samples,nsteps=0)

        # multiprocessing suffices
        dissonance_array = Cg.dissonance_multiple(samples)
        # multiprocessing suffices
        dataframes,error_array = Cg.randomMaskReconstruction_multiple(samples)
        # multiprocessing suffices
        ideology_index = Cg.ideology_multiple(samples)
        # multiprocessing suffices
        local_dispersion = Cg.dispersion_multiple(samples)
        # compute distance from each pole
        # multiprocessing suffices
        array_distances = Cg.polarDistance_multiple(samples)
        #-------------------



import sys
sys.path.append("../cognet")

from quasinet.qnet import qdistance
from cognet import cognet

from dataFormatter import dataFormatter
from model import model
import pandas as pd
import numpy as np
import dill as pickle

yr = '2016'
POLEFILE='examples_data/polar_vectors.csv'
QPATH='../../creed2_/GSS/Qnets/gss_'+yr+'.joblib'
IMMUTABLE_FILE='examples_data/immutable.csv'
GSSDATA = 'examples_data/processed_data/gss_'+yr+'.csv'

commandline_test = True
#train data object
data = dataFormatter(samples=GSSDATA,
                test_size=0.5)
features,samples = data.train()

im_vars_df = pd.read_csv(IMMUTABLE_FILE, names=['vars'])
im_vars_list = im_vars_df.vars.to_list()
mutable_vars, immutable_vars = data.mutable_variables(immutable_list=im_vars_list)
mutable_vars, immutable_vars = data.mutable_variables(IMMUTABLE_FILE=IMMUTABLE_FILE)

model_ = model()
model_.load(QPATH)


#comm = MPI.COMM_WORLD
#rank = comm.rank
#if rank == 0:
#    Cg = cognet()
#    Cg.load_from_model(model_, samples_file='examples_data/gss_2018.csv')
#    stats = Cg.set_poles(POLEFILE,steps=2)
#    Cg.num_qsamples = 5
#    Cg.set_nsamples(10)
#else:
#    Cg = None
#
#Cg = comm.bcast(Cg, root=0)

Cg = cognet()
Cg.load_from_model(model_, samples_file=GSSDATA)
#
#### produce stats on how many column names actually match
stats = Cg.set_poles(POLEFILE,steps=2)
Cg.num_qsamples = 5
Cg.set_nsamples(500)

with open('examples_results/cgmod' + yr + '.mod', 'wb') as f:
    pickle.dump(Cg,f)
f.close()





# This is an example of how we see 
# the package work. The functions listed here
# are probably teh only ones that should be exposed, ie documented.
# others should br prepended with a double underscore
#  
# The cognet directory has the following "modules"
# which are seprate .py files containing clases and functions
import sys
sys.path.append("../cognet")

from quasinet.qnet import qdistance
from cognet import cognet

from dataFormatter import dataFormatter
from model import model
import pandas as pd
import numpy as np

yr = '2018'
POLEFILE='examples_data/polar_vectors.csv'
QPATH='examples_data/gss_'+yr+'.joblib'
IMMUTABLE_FILE='examples_data/immutable.csv'
GSSDATA = 'examples_data/gss_'+yr+'.csv'

yr=sys._xoptions['y']
Nstr=sys._xoptions['N']
NAME_PREF='../ideology_2/ALL_Qdist_Matrices/ALL_DMAT_test'
if Nstr != 'all':
    N=int(Nstr)
else:
    N=None

#train data object
data = dataFormatter(samples=GSSDATA,
                test_size=0.5)
features,samples = data.train()

im_vars_df = pd.read_csv(IMMUTABLE_FILE, names=['vars'])
im_vars_list = im_vars_df.vars.to_list()
mutable_vars, immutable_vars = data.mutable_variables(immutable_list=im_vars_list)
mutable_vars, immutable_vars = data.mutable_variables(IMMUTABLE_FILE=IMMUTABLE_FILE)

model_ = model()
model_.load("examples_data/gss_2018.joblib")

Cg = cognet()
Cg.load_from_model(model_, samples_file='examples_data/gss_2018.csv')

# get features for each year
cols = (pd.read_csv(GSS_FEATURES_BY_YEAR,
                               keep_default_na=True, 
                               index_col=0).set_index(
                                   'year')).loc[int(yr)].apply(
                                       eval).values[0]
#read sample data
samples_=pd.read_csv(GSSDATA)
if N is not None:
    P=samples_.sample(N)
else:
    P=samples_
    
P_=P#[[x for x in poles.columns if x in P.columns]]
# for x in poles.columns:
#     if x not in P_.columns:
#         P_[x]=np.nan

#cols = cols_by_year[yr]
features = pd.DataFrame(columns=cols)
all_features = pd.concat([poles, features], axis=0)
pL= all_features.loc['L'][cols].fillna('').values.astype(str)[:]
pR= all_features.loc['R'][cols].fillna('').values.astype(str)[:]
p_all=pd.concat([P_, features], axis=0)[cols].fillna('').values.astype(str)[:]

w = P.index.size
h = w

def distfunc(x, y):
    d=qdistance(x,y,qnet,qnet)
    return d

def dfunc_line(k):
    '''
       args:
          k (int): row
      return:
          numpy.ndarray(float)
    '''
    line = np.zeros(w)
    y = p_all[k]
    for j in range(w):
        if j > k:
            x = p_all[j]
            line[j] = distfunc(x, y)
    return line

if __name__ == '__main__':

    with MPIPoolExecutor() as executor:
        result = executor.map(dfunc_line, range(h))
        pd.DataFrame(result).to_csv(NAME_PREF+yr+'.csv',index=None,header=None)


# This is an example of how we see 
# the package work. The functions listed here
# are probably teh only ones that should be exposed, ie documented.
# others should br prepended with a double underscore
#  
# The cognet directory has the following "modules"
# which are seprate .py files containing clases and functions
from mpi4py.futures import MPIPoolExecutor
from mpi4py import MPI

import sys
sys.path.append("../../cognet")

from quasinet.qnet import qdistance
from cognet import cognet

from dataFormatter import dataFormatter
from model import model
import pandas as pd
import numpy as np
import dill as pickle

comm = MPI.COMM_WORLD
size = comm.Get_size()
rank = comm.Get_rank()

if rank == 0:
    yr = '2016'
    with open('../examples_results/cgmod'+yr+'.mod', 'rb') as f:
        Cg = pickle.load(f)
    print("loaded")
    data = Cg
else:
    data = None
data = comm.scatter(data, root=0)

if __name__ == '__main__':

    with MPIPoolExecutor() as executor:
        print("MPI starting")
        result = executor.map(Cg.distfunc_line, range(len(Cg.samples)))
        print(result)
        pd.DataFrame(result).to_csv('../examples_results/distfunc_mpitest_preloadedclass_computenode_2016.csv',index=None,header=None)



import os
import sys
import subprocess as sp

run = ''

py2 = sys.version_info[0] < 3

run_file = sys.argv[1]
dataset = sys.argv[2]
assert ".py" in run_file

if py2:
    #change python version if needed
    py_load = 'python/3.7.0'
    py_ver='python3'
else:
    py_load = 'python/3.7.0' #'python/intel-2020.up1' load version of python
    py_ver='python'
    
print('Running with MPI ')
py_ver='mpirun python3 -m mpi4py.futures'
run = run + "_MPI"
t_per_node = 28

first ="""#!/bin/bash 
#SBATCH --job-name={args:.4}{run:.4}{dataset:.4}
#SBATCH --output=./examples_results/{args:.4}{run:.4}{dataset:.4}.out
#SBATCH --error=./examples_results/{args:.4}{run:.4}{dataset:.4}.err
#SBATCH --time=8:00:00
#SBATCH --qos=normal
#SBATCH --nodes=4
#SBATCH --exclusive
#SBATCH --ntasks-per-node={t_per_node}
#SBATCH --partition=broadwl
date; 
module load midway2
module unload python
module load python/anaconda-2020.02
module unload python
module load python/cpython-3.7.0
{py_ver} {args};
date; """.format(dataset=dataset,py_load=py_load, py_ver=py_ver, run=run, t_per_node=t_per_node, args=' '.join(sys.argv[1:]))


name_file ="examples_results/quick_runab.sbc"
with open(name_file, 'w') as f:
    f.write(first)   
sp.call('sbatch '+name_file,shell=True)
sp.call('rm '+name_file,shell=True)
#format: python submit_one.py my_code.py job_name